import time
from selenium import webdriver
from bs4 import BeautifulSoup
from quick_crawler import page

def get_coutries_list(chrome_driver_path,save_path="datasets/countries.csv",temp_html_file="datasets/html.txt"):
    options = webdriver.ChromeOptions()
    # options.add_argument("headless")
    # set chromedriver.exe's path
    driver = webdriver.Chrome(executable_path=chrome_driver_path,
                              # chrome_options=options
                              )
    driver.implicitly_wait(0.5)
    # launch the page
    driver.get("https://www.nationsonline.org/oneworld/countries_of_the_world.htm")

    html_obj = driver.find_element_by_tag_name("html")
    f_out = open(temp_html_file, "w", encoding='utf-8')
    f_out.write(html_obj.get_attribute("outerHTML"))
    f_out.close()

    time.sleep(10)

    html = BeautifulSoup(html_obj.get_attribute("outerHTML"), features='lxml')
    # find elements
    # table_obj = driver.find_element_by_tag_name("table")
    # obtain target htmlstr
    # print(table_obj.get_attribute("outerHTML"))

    tables = html.findAll("table")

    print(tables)

    driver.close()

    list_model = []
    for table in tables:
        trs = table.findAll("tr")
        for tr in trs:
            # print(tr)
            list_v = []
            for idx, td in enumerate(tr.findAll("td")):

                if idx == 0:
                    flag = td.find("div", {"class": "flag"})
                    if flag == None:
                        break
                    id = flag["id"]

                    list_v.append(id)
                elif idx == 1:
                    if td.find("a") == None:
                        break
                    href = td.find("a")["href"]
                    list_v.append('https://www.nationsonline.org/oneworld/' + href)
                    list_v.append(td.text)
                else:
                    list_v.append(td.text)
            print(list_v)
            if len(list_v) < 5:
                continue
            model = {
                "Id": list_v[0],
                "Url": list_v[1],
                "English Name": page.quick_remove_unicode(list_v[2]),
                "French Name": page.quick_remove_unicode(list_v[3]),
                "Local Name": page.quick_remove_unicode(list_v[4]),
                "Region": page.quick_remove_unicode(list_v[5])
            }
            list_model.append(model)

    page.quick_save_csv(save_path, ['Id', 'Url', 'English Name', 'French Name', 'Local Name', 'Region'], list_model)







import pickle, json, requests
import numpy as np
import jieba
import os
from tqdm import tqdm
import pickle
from quick_crawler.page import *
import jieba.posseg as peg
from bs4 import BeautifulSoup
from wordcloud import WordCloud
import matplotlib.pyplot as plt

def load_all_data(source_csv_file,raw_data_folder,keywords_path="",keyword_file_list="",keyword_file_list_folder="",save_corpus_path=""):
    # load all data
    corpus = []
    list_model = quick_read_csv_model(source_csv_file)

    if keywords_path!="" and keyword_file_list!="":
        carbon2_dictionary = load_high_quality_dictionary(keyword_file_list)
        pickle.dump(carbon2_dictionary, open(keywords_path, "wb"))
        print(carbon2_dictionary)
        jieba.load_userdict(carbon2_dictionary)

    for model in tqdm(list_model):
        file_id = model["FileId"]
        file_path = f"{raw_data_folder}/{file_id}.txt"
        if os.path.exists(file_path):
            html_obj = BeautifulSoup(open(file_path, 'r', encoding='utf-8').read(), features='lxml')
            remove_tags = ['style', 'script', 'svg', 'path', 'noscript']
            for tag in remove_tags:
                for item in html_obj.findAll(tag):
                    item.decompose()
            full_text = ""
            ps = html_obj.findAll("p")
            for p in ps:
                p_text = p.text
                p_text = p_text.strip()
                if len(p_text) > 10:
                    full_text += p_text + "\n"
            full_text = full_text.strip()
            full_text = ' '.join(jieba.cut(full_text))
            # print(full_text)
            tokens = process_text_into_tokens(full_text)
            print(file_path, tokens)
            corpus.append(tokens)

    pickle.dump(corpus, open(save_corpus_path, "wb"))
    return corpus

def load_high_quality_dictionary(file_list="cnki_data.txt;cnki_data1.txt;cnki_data2.txt",folder="datasets"):
    files=file_list.split(";")
    list_words = []
    for f in files:
        lines=open(f"{folder}/{f}","r",encoding='utf-8')
        for l in lines:
            l=l.strip()
            if l.startswith("K1"):
                ks=l.split(" ")
                if len(ks)==2:
                    ws=ks[1].split(";")
                    print(ws)
                    for k in ws:
                        if k not in list_words:
                            list_words.append(k)
    return list_words



def load_stopwords(stopwords_folder="datasets/stopwords"):
    list_stopwords_all=[]
    for file in os.listdir(stopwords_folder):
        path=os.path.join(stopwords_folder,file)
        lines=open(path,'r',encoding='utf-8').readlines()
        for w in lines:
            w=w.strip()
            if w!="":
                if w not in list_stopwords_all:
                    list_stopwords_all.append(w)
    return list_stopwords_all

list_stopwords=load_stopwords()

def seg_cut(in_str):
    words = peg.cut(in_str)
    result1 = ""
    list_word=[]
    for word, flag in words:
        # temp = "%s_%s " % (word, flag)
        # result1 = result1 + temp
        list_word.append([word,flag])
    return list_word

def process_text_into_tokens(text):
    text=text.strip()
    lines=text.split("\n")
    tokens=[]
    for line in lines:
        line=line.strip()
        sentences=line.split("。")
        for sentence in sentences:
            sentence=sentence.strip()
            # words=jieba.cut(sentence,cut_all=False)
            words=seg_cut(sentence)
            new_words=[]
            for item in words:
                w=item[0]
                pos=item[1]
                w=w.strip()
                w = w.lower()
                if w=="":
                    continue
                if len(w)!=4 and w.isnumeric():
                    continue
                if len(w)==4 and w.isnumeric() and not w.startswith("20") and not w.startswith("19"):
                    continue
                if pos not in ['n']: # 词性过滤
                    continue
                if not w in list_stopwords:
                    new_words.append(w)

            for w in new_words:
                tokens.append(w)
    return tokens



class KeyWordsBasesOnTermFreq():

    def get_freq(self, words):  # 统计一个词语列表中的term freq
        word_freq_map = {}
        for word in words:
            word_freq_map[word] = word_freq_map.get(word, 0) + 1
        return word_freq_map

    # 基于词语频数/term freq抽取关键词
    def get_keywords_based_on_freq(self, content, topN=10):
        words = process_text_into_tokens(content)  # 分词
        word_freq_map = self.get_freq(words)  # 获取词频
        keywords = sorted(word_freq_map.items(), key=lambda x: x[1], reverse=True)  # 按照词频倒序排列
        keywords = keywords[:topN]  # 挑选关键词
        keywords = list(map(lambda x: x[0], keywords))
        return keywords


class KeyWOrdsBasedOnTFIDF(KeyWordsBasesOnTermFreq):

    def __init__(self, corpus=None, idf_path=None):
        self.init_idf(corpus, idf_path)

    def init_idf(self, corpus, idf_path):
        # 初始化idf.如果制定了文件路径就直接加载；如果没有指定就训练一个
        if idf_path == None:

            words_list = corpus
            doc_num = len(words_list)
            doc_freq_map = {}
            for words in words_list:
                for word in set(words):
                    doc_freq_map[word] = doc_freq_map.get(word, 0) + 1
            self.idf_map = {}
            for word in doc_freq_map:
                self.idf_map[word] = np.log(doc_num / (doc_freq_map[word] + 1) ** 0.6)
            print(self.idf_map)
            # pickle.dump(self.idf_map, open('idf.pkl', 'wb'))
        else:
            self.idf_map = pickle.load(open(idf_path, 'rb'))

    def get_tfidf(self, content):
        words = process_text_into_tokens(content)
        tf_map = self.get_freq(words)
        tfidf_map = {}
        for word in tf_map:
            if word in self.idf_map:  # 训练语料中没有出现的词语就干掉了
                tfidf_map[word] = tf_map[word] * self.idf_map[word]
        return tfidf_map

    def get_keywords_based_on_tfidf(self, content, topN=10):
        tfidf_map = self.get_tfidf(content)  # 获取tfidf值
        keywords = sorted(tfidf_map.items(), key=lambda x: x[1], reverse=True)
        keywords = keywords[:topN]
        keywords = list(map(lambda x: x[0], keywords))
        return keywords

def get_all_tokens(corpus):
    all_text=[]
    for tokens in corpus:
        all_text+=tokens
    return all_text

def get_all_text(corpus):
    all_text=""
    for tokens in corpus:
        all_text+=''.join(tokens)
    return all_text

def run_tfidf_model(source_csv_file,raw_data_folder,keywords_path="",keyword_file_list="",keyword_file_list_folder="",save_corpus_path="",save_freq_filepath="",save_tfidf_path=""):

    corpus=load_all_data(source_csv_file,raw_data_folder,keywords_path,keyword_file_list,keyword_file_list_folder, save_corpus_path)

    kw_extractor_tf = KeyWordsBasesOnTermFreq()
    print("Term Frequency: ")
    kw_extractor_tf.get_freq(get_all_tokens(corpus))
    keywords=kw_extractor_tf.get_keywords_based_on_freq(get_all_text(corpus), topN=100)
    pickle.dump(keywords,open(save_freq_filepath,"wb"))
    for w in keywords:
        print(w)
    print()
    print("TF-IDF ：")
    kw_extractor_tfidf = KeyWOrdsBasedOnTFIDF(corpus=corpus)
    dict_words={}
    for c in corpus:
        list_words=kw_extractor_tfidf.get_keywords_based_on_tfidf(''.join(c),topN=20)
        for w in list_words:
            if w in dict_words:
                dict_words[w]+=1
            else:
                dict_words[w]=1
    sorted_dict_words=sorted(dict_words.items(), key = lambda kv:(kv[1], kv[0]), reverse=True)
    pickle.dump(sorted_dict_words,open(save_tfidf_path,"wb"))
    print("Term\tFrequency")
    count=0
    for word, freq in sorted_dict_words[:100]:
        if count >= 100:
            break
        print(f"{word}\t{freq}")
        count += 1


def export_keywords(freq_file, tfidf_file, save_tfidf_keywords, save_tf_keywords):
    keywords_freq = pickle.load(open(freq_file, 'rb'))
    for k in keywords_freq:
        print(k)

    print()

    keywords_tfidf = pickle.load(open(tfidf_file, "rb"))
    print(keywords_tfidf)
    count = 0
    for word, freq in keywords_tfidf[:100]:
        if count >= 100:
            break
        print(f"{word}\t{freq}")
        count += 1

    f_out = open(save_tfidf_keywords, "w", encoding='gbk')
    f_out.write("Keyword(TF),Keywords(TF-IDF)\n")
    for idx, k in enumerate(keywords_freq):
        f_out.write(f"{k},{keywords_tfidf[idx][0]}\n")
    f_out.close()

def load_cnki_dictionary(file_list="cnki_data.txt;cnki_data1.txt;cnki_data2.txt",folder="datasets"):
    files=file_list.split(";")
    dict_words={}
    for f in files:
        lines=open(f"{folder}/{f}","r",encoding='utf-8')
        for l in lines:
            l=l.strip()
            if l.startswith("K1"):
                ks=l.split(" ")
                if len(ks)==2:
                    ws=ks[1].split(";")
                    print(ws)
                    for k in ws:
                        if k.strip()=="":
                            continue
                        if k not in dict_words:
                            dict_words[k]=1
                        else:
                            dict_words[k]+=1
    return dict_words

def export_keywords_dictionary(file_list="cnki_data.txt;cnki_data1.txt;cnki_data2.txt",folder="datasets",save_csv_path=""):
    dict_words_cnki = load_cnki_dictionary(file_list,folder)
    sorted_dict_words_cnki = sorted(dict_words_cnki.items(), key=lambda kv: (kv[1], kv[0]), reverse=True)
    f_out = open(save_csv_path, "w", encoding='gbk')
    f_out.write("Keyword,Frequency\n")
    for t, f in sorted_dict_words_cnki:
        f_out.write(f"{t},{f}\n")
    f_out.close()


def load_corpus(corpus_path,remove_words=""):

    corpus=pickle.load(open(corpus_path,"rb"))

    all_text=[]
    for tokens in corpus:
        new_tokens=[]
        for w in tokens:
            if remove_words!="":
                wl=remove_words.split(";")
                if w not in wl:
                    new_tokens.append(w)
            else:
                new_tokens.append(w)

        all_text=all_text + new_tokens

    return all_text

def run_wordcloud(corpus_path,save_fig_path, font_path):
    # list_word=load_cnki_dictionary()
    list_word = load_corpus(corpus_path)

    print(len(list_word))
    mytext = ' '.join(list_word)
    current_path = os.path.dirname(os.path.realpath(__file__))
    wordcloud = WordCloud(
        max_words=150,
        # font_path=f"{current_path}/utils/fonts/SimHei.ttf",
        font_path=font_path,
        width=2000, height=2000, background_color="white",
    ).generate(mytext)

    plt.imshow(wordcloud, interpolation='bilinear')
    plt.axis("off")
    plt.savefig(save_fig_path, dpi=600)
    plt.show()



import hashlib
import os

from quick_crawler.browser import *
import uuid
from carbon2.api.submit import Carbon2Api
from tqdm import tqdm

def get_page_meta(html_str):
    try:
        soup = BeautifulSoup(html_str, features="lxml")
        title=""
        keywords=""
        description=""

        if soup!=None and soup.title!=None:
            title = soup.title.string


        # print(soup.attrs)
        html=soup.find("html")
        if "lang" in html.attrs.keys():
            lang = html["lang"]
        else:
            lang = ""
        # print("lang = ",lang)

        meta = soup.find_all('meta')
        # print(html_str)
        for tag in meta:
            if 'name' in tag.attrs.keys():
                name=tag.attrs['name'].strip().lower()
                if name=="description":
                    if 'content' in tag.attrs.keys():
                        description=tag.attrs['content']
                    else:
                        description=""
                if name=="keywords":
                    keywords=tag.attrs['content']
        model = {
            "title":title.replace("\n",""),
            "lang":lang,
            "keywords":keywords.replace("\n",""),
            "description":description.replace("\n","")
        }
    except:
        model = {
            "title": "",
            "lang": "",
            "keywords": "",
            "description":""
        }
    print("【Meta】")
    print('title = ', model["title"])
    print('keywords = ', model["keywords"])
    print('description = ', model["description"])
    print('language = ', model["lang"])
    return model

def upload_to_server(server_url,user_id,target_url,driver_path="",save_folder="html_data",use_md5url_as_id=False,tag="",language="",publishtime=""):
    print("==========Begin Upload===============")
    print("UserId: "+user_id, ", Target Url:"+target_url)

    if driver_path=="" or not os.path.exists(driver_path):
        print("Chromedriver.exe is not set due to the invalid driver path; use conventional crawler instead. ")

    # carbon2system's url
    root_url = f"{server_url}/api"
    c2api = Carbon2Api(root_url)

    # the page url that you want to upload
    # target_url="http://xinhuanet.com/"

    # 1. Generate an unique ID
    page_id=""
    if use_md5url_as_id:
        unique_id=hashlib.md5(target_url.encode())
        page_id=str(unique_id.hexdigest())
    else:
        unique_id = uuid.uuid4()
        page_id=str(unique_id)

    print(f"Using ID: {page_id}...")

    # check if exists
    if c2api.exists_url(target_url)==1:
        print(f"Url exists: {target_url}, skip the url.")
        return

    # 2. quick obtain an HTML page
    if driver_path=="" or not os.path.exists(driver_path):
        html_str=quick_html_page(target_url)
    else:
        html_str=get_html_str_with_browser(url=target_url,driver_path=driver_path,slient=True)
    if html_str=="":
        print("getting html page occurs error, empty html string, skip uploading")
        return

    if not os.path.exists(save_folder):
        print("Creating the folder: ", save_folder)
        os.mkdir(save_folder)
    saved_file_path=f"{save_folder}/{page_id}.txt"
    print("Saved file path: ", saved_file_path)
    f_out=open(saved_file_path,"w",encoding="utf-8")
    f_out.write(html_str)
    f_out.close()

    # 2.1 get meta info
    meta_model = get_page_meta(html_str)

    # 3. submit the meta data

    r=c2api.submit_metadata(target_url,meta_model["title"],user_id,keywords=meta_model["keywords"],description=meta_model["description"], file_id=page_id,tag=tag,language=language,publishtime=publishtime)

    if r==1:
        # 4. submit the file with same unique id
        r=c2api.submit_file(f"{save_folder}/{page_id}.txt")

        # 5. Verify if upload success
        download_url=f"{server_url}/WebData/{page_id}.txt"
        if check_url_ok(download_url):
            print("Upload successfully")
    else:
        print("Upload error: the url may be repeated!")
    print("==========End Upload===============")

# the csv file must contain fields real_url, title.
def submit_page_list(server_url, user_id,csv_file,save_html_folder, use_md5url_as_id=False, driver_path="browsers/chromedriver.exe",tag="",language="",url_field_name="real_url",publishtime="",try_raise_error=False):
    list_model = quick_read_csv_model(csv_file, encoding='utf-8')
    for idx,model in enumerate(list_model):
        print(f"{idx + 1}/{len(list_model)}")
        url = model[url_field_name]
        if try_raise_error:
            upload_to_server(server_url, user_id, url, use_md5url_as_id=use_md5url_as_id, driver_path=driver_path,
                             tag=tag, language=language, save_folder=save_html_folder, publishtime=publishtime)
        else:
            try:
                upload_to_server(server_url,user_id, url,use_md5url_as_id=use_md5url_as_id, driver_path=driver_path,tag=tag,language=language,save_folder=save_html_folder,publishtime=publishtime)
                print()
            except Exception as err:
                print("【ERROR】")
                print("Error Info: ",err)
                print()

def submit_url_list(server_url, user_id,list_url,use_md5url_as_id=False, driver_path="",tag="",language="",save_html_folder="html_data",publishtime=""):
    for idx,url in enumerate(list_url):
        try:
            print(f"{idx+1}/{len(list_url)}")
            upload_to_server(server_url,user_id, url,use_md5url_as_id=use_md5url_as_id, driver_path=driver_path,tag=tag,language=language,save_folder=save_html_folder,publishtime=publishtime)
            print()
        except:
            print("Error in calling API!")
            print()




import time

import requests
import json
from tqdm import tqdm
from quick_crawler.page import quick_html_page,quick_html_object,quick_download_file,quick_save_csv
from quick_crawler.browser import get_html_str_with_browser
import os

def get_page_count_from_server(server_url,page_size,uploader,tag,use_fields):
    api_url = f"{server_url}/api/Data"
    parameters = {
        "PageIndex": 1,
        "PageSize": page_size
    }

    if uploader != "":
        parameters["Uploader"] = uploader
    if tag != "":
        parameters["Tag"] = tag
        parameters["Fields"] = use_fields

    r = requests.get(api_url, params=parameters)
    page = json.loads(r.text)
    # print(page)
    print("PageCount:", page["PageCount"])
    return page["PageCount"]

def download_data_from_server(server_url,page_size,save_csv_path="",save_rawdata_folder="",func_process=None,uploader="",tag="",use_fields="*",sleep_time=-1,server_timeout=20,func_download_before=None):

    api_url = f"{server_url}/api/Data"
    list_model = []
    fields = []
    page_count=get_page_count_from_server(server_url,page_size,uploader,tag,use_fields)
    print(f"Downloading raw data to: {save_rawdata_folder}")
    for page_index in tqdm(range(1,page_count+1)):# page count
        parameters = {
            "PageIndex": page_index,
            "PageSize": page_size
        }
        if uploader!="":
            parameters["Uploader"]=uploader
        if tag!="":
            parameters["Tag"]=tag
            parameters["Fields"]=use_fields

        r = requests.get(api_url, params=parameters)
        page = json.loads(r.text)
        print(page)
        # print(page)
        # print("Page index = ",page_index)
        # print("PageCount:", page["PageCount"])
        if page==None or page["DataTable"]==None:
            continue
        for row in page["DataTable"]:
            # print(row)
            if len(fields) == 0:
                fields = row.keys()
                # print(fields)
            file_id = row["FileId"]
            download_fulltext_url = f"{server_url}/WebData/{file_id}.txt"
            if save_rawdata_folder!="":
                if not os.path.exists(save_rawdata_folder):
                    os.mkdir(save_rawdata_folder)
                save_path = f"{save_rawdata_folder}/{file_id}.txt"
                if not os.path.exists(save_path):
                    try:
                        if func_download_before!=None:
                            if func_download_before(server_url,file_id)==True:
                                print("downloading...")
                                quick_download_file(download_fulltext_url, save_file_path=save_path)
                            else:
                                continue
                        else:
                            quick_download_file(download_fulltext_url, save_file_path=save_path)
                        if func_process!=None:
                            func_process(row,open(save_path,"r",encoding='utf-8').read())
                    except:
                        print("error in downloading file", download_fulltext_url)
            else:
                html_str=quick_html_page(download_fulltext_url,timeout=server_timeout)
                func_process(row,html_str)
            if save_csv_path!="":
                list_model.append(row)
        if sleep_time!=-1:
            time.sleep(sleep_time)
    if save_csv_path!="":
        print("Saving meta data file to: ",save_csv_path)
        quick_save_csv(save_csv_path, field_names=fields, list_rows=list_model)
    print("Finished!")
